<?php
use Teknisa\Model\Gestaopessoas\Entities\EsoRegistro;
use Teknisa\Model\Gestaopessoas\Entities\EsoEvento;
use Teknisa\Model\Fiscal\Entities\Imposto;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\DateUtil;
use Zeedhi\Framework\Session\Session;

require_once __DIR__ .DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'fis'.'/backend/bootstrap.php';

function main() 
{

    ini_set('memory_limit','2048M');

    $leiaute_path = __DIR__."/files";
    
    //******************************************//
    // Alterar de acordo com a versão desejada *//
    //******************************************//    
    
    $versao  = "";
    $arquivo = ""; //Nome do arquivo xls com os layouts da versao
    $nrorg   = "";
    
    if (!is_file($leiaute_path."/".$arquivo)) {
        echo "Arquivo $arquivo não encontrado.\n";
        die;
    } else {
        echo "Lendo arquivo $arquivo...\n";
    }

    $handle = new ManipulacaoExcel($leiaute_path."/".$arquivo);
    $info = $handle->preparaDados();
    echo "Dados das planilhas lidos com sucesso!!\n";
    echo "Gravando dados...\n";
    $persist = new Persistencia($info, $nrorg, $versao);
    $persist->persisteTabelas($versao);

    $result = "Leiautes gerados com sucesso!!!\n";

    echo $result;
}

class ManipulacaoExcel
{
    private $file;

    private $eventos;
    private $resumos;
    private $registros;

    private $leiaute;

    private $lista_eventos;

    public function __construct($arquivo)
    {
        $this->file = \PHPExcel_IOFactory::load($arquivo);

        $this->eventos = array();
        $this->resumos = array();
        $this->registros = array();

        $this->leiaute = array();

        $this->lista_eventos = array();
    }

    public function preparaDados()
    {
        try {
            self::carregaEventos();
            self::carregaRegistros();
            
            $this->leiaute = array(
                'EVENTOS' => $this->eventos,
                'REGISTROS' => $this->registros
            );
            return $this->leiaute;
        } catch (\Exception $e) {
            throw new \Exception("hrm_nenhum_dado_encontrado");
        }
    }

    private function carregaEventos()
    {
        try {
            $this->file->setActiveSheetIndexByName('Eventos');

            for ($row = 2; !is_null($this->file->getActiveSheet()->getCell('A'.$row)->getValue()); $row++) {
                $nmevento = $this->file->getActiveSheet()->getCell('A'.$row)->getValue();
                $grupo = $this->file->getActiveSheet()->getCell('B'.$row)->getValue();

                $this->eventos[] = array(
                    'DESCRICAO' => $nmevento,
                    'NOME' => substr($nmevento, 0, 6),
                    'IDGRUPEVENT' => $grupo
                );
                $this->lista_eventos[] = substr($nmevento, 0, 6);
            }
        } catch (\Exception $e) {
            throw Exception::carregaEventos();
        }
    }

    private function carregaRegistros()
    {
        try {
            $this->file->setActiveSheetIndexByName('Registros');
            $activeSheet = $this->file->getActiveSheet();
            $listaEventos = $this->lista_eventos;

            for ($row = 1; !empty($listaEventos); $row++) {
                $value = $activeSheet->getCell('A'.$row)->getValue();

                /*Extrai o nome do evento do título da tabela (Ex: Registros do evento S-4999 - Adesão antecipada ao eSocial)*/
                $eventoIndex = array_search(substr($value,20,6), $listaEventos);
                if ($eventoIndex !== false) {
                    $nmevento = $listaEventos[$eventoIndex];
                    $endRow = self::percorreRegistroEvento($activeSheet, $nmevento, $row);
                    $row = $endRow;

                    unset($listaEventos[$eventoIndex]);
                }
            }
        } catch (\Exception $e) {
            throw Exception::carregaRegistros();
        }
    }

    private function percorreRegistroEvento($activeSheet, $nmevento, $eventoRow)
    {
        /*Pula as linhas com o nome do evento e os nomes das colunas*/
        $startRow = $eventoRow + 2;

        for ($row=$startRow; !is_null($activeSheet->getCell('A'.$row)->getValue()); $row++) {

            $nrCampo = $activeSheet->getCell('A'.$row)->getValue();
            $this->registros[$nmevento][] = array(
                'NRCAMPO' => $activeSheet->getCell('A'.$row)->getValue(),
                'CAMPO' => $activeSheet->getCell('B'.$row)->getValue(),
                'PAI' => $activeSheet->getCell('C'.$row)->getValue(),
                'ELEMENTO' => $activeSheet->getCell('D'.$row)->getValue(),
                'TIPO' => $activeSheet->getCell('E'.$row)->getValue(),
                'OCORRENCIA' => $activeSheet->getCell('F'.$row)->getValue(),
                'TAMANHO' => $activeSheet->getCell('G'.$row)->getValue(),
                'DECIMAL' => $activeSheet->getCell('H'.$row)->getValue(),
                'DESCRICAO' => $activeSheet->getCell('I'.$row)->getValue()
            );
            $lastRow = $row;
        }

        return $lastRow;
    }
}

class Persistencia
{
    private $eventos;
    private $resumos;
    private $registros;
    private $nrorgpadrao;

    /**
     * @param $caminho          Pasta na qual se encontram os arquivos de leiaute
     * @param $versao           Versão do E-Social
     * @param $arq_eventos      Nome do arquivo com a descrição dos eventos
     * @param $arq_resumos      Nome do arquivo com os resumos dos eventos
     * @param $arq_registros    Nome dos arquivos com os registros do eventos
     * @throws \Exception
     */
    public function __construct($leiautes, $nrorg, $versao)
    {
        try {
            $this->validaNrEventos($leiautes, $nrorg,$versao);
            $this->eventos = $leiautes['EVENTOS'];
            $this->registros = $leiautes['REGISTROS'];
            $this->nrorgpadrao = $nrorg;
        } catch (\Exception $e) {
            throw $e;
            
        }
    }

    public function persisteTabelas($versao)
    {
        try {
            $connection = Setforms2\Shared\Registry::getEntityManager()->getConnection();
            $connection->beginTransaction();
            
            self::insereEvento($versao, $connection);
            self::geraArvoreRegistro($versao,$connection);
            
            $connection->commit();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Validada se a quantidade de eventos em cada leiaute está consistente.
     *
     * @param $leiautes
     *
     * @return bool
     * @throws \Exception
     */
    private function validaNrEventos($leiautes, $nrorg, $versao)
    {
        try {
            $nrEventos = count($leiautes['EVENTOS']);
            $nrRegistros = count($leiautes['REGISTROS']);

            if (($nrEventos != $nrRegistros)) {
                throw new \Exception("Numero de eventos diferentes do numero de registros");
            }
            
            $connection = Setforms2\Shared\Registry::getEntityManager()->getConnection();
            // Verifica se os Eventos  ja estão inseridos para a versão informada
            foreach ($leiautes['EVENTOS'] as $evento) {
                $params = array(
                    "NMEVENTO" => $evento["NOME"],
                    "NRORG"    => $nrorg,
                    "DSVERSAO" => $versao
                );
                $sql = "SELECT * FROM EVENTOREINF WHERE NMEVENTO = :NMEVENTO AND DSVERSAO =:DSVERSAO AND NRORG =:NRORG";
                $std = $connection->prepare($sql);
                $std->execute($params);
                $resul =  $std->fetch(); 
                if($resul)
                    throw new \Exception("Os eventos ja estão inseridos nessa versao");
                
            }
            

            
            
        } catch (\Exception $e) {
            throw $e;
            
        }
    }

    private function insereEvento($versao, $connection)
    {
        try {
            $newCode     = new  NewCode(\Setforms2\Shared\Registry::getEntityManager()->getConnection());
            foreach ($this->eventos as $evento) {
                
                $nrEvento = $newCode->getCodeByTableName('EVENTOREINF', $this->nrorgpadrao );
                //var_dump($nrEvento);die;
                $params = array(
                    "NREVENTO"     => $nrEvento,
                    "DSEVENTO"     => $evento['DESCRICAO'],
                    "NMEVENTO"     => $evento['NOME'],
                    "IDGRUPO"      => $evento['IDGRUPEVENT'],
                    "DSVERSAO"     => $versao,
                    "NRORG"        => '20'
                );
                
                $sql = "INSERT INTO EVENTOREINF
                (NREVENTO, DSEVENTO, NMEVENTO, IDGRUPO,  DSVERSAO, NRORG)
                        VALUES
                (:NREVENTO,:DSEVENTO,:NMEVENTO,:IDGRUPO, :DSVERSAO,:NRORG)";
        
                $std = $connection->prepare($sql);
                $std->execute($params);
                
                \Setforms2\Shared\Registry::getEntityManager()->flush();
                
                self::insereRegistroEvento($nrEvento, $this->registros[$evento['NOME']], $connection);
            }

            
         }catch (\Exception $e) {
            var_dump($e->getMessage());die;
            throw $e;
            
        }
    }

    private function insereRegistroEvento($nrEvento, $registros, $connection)
    {
        try {
            $newCode     = new  NewCode(\Setforms2\Shared\Registry::getEntityManager()->getConnection());
            
            foreach ($registros as $registro) {
                $nrRegistro = $newCode->getCodeByTableName('REGEVENREINF',$this->nrorgpadrao);
                $params = array(
                    "NRCAMPO"      => $registro['NRCAMPO'],
                    "NRREGISTRO"   => $nrRegistro,
                    "NREVENTO"     => $nrEvento,
                    "NMCAMPO"      => preg_replace('~[\n]+~','',$registro['CAMPO']),
                    "NMCAMPOPAI"   => preg_replace('~[\n]+~','',$registro['PAI']),
                    "DSELEMENTO"   => $registro['ELEMENTO']==="CG" ? "G" : $registro['ELEMENTO'],
                    "DSTIPO"       => $registro['TIPO'],
                    "DSOCORRENCIA" => $registro['OCORRENCIA'],
                    "NRTAMANHO"    => $registro['TAMANHO']==="-" ? null : $registro['TAMANHO'],
                    "NRDECIMAIS"   => $registro['DECIMAL']==="-" ? null : $registro['DECIMAL'],
                    "DSREGISTRO"   => preg_replace('~[\n]+~',' ',$registro['DESCRICAO']),
                    "NRORG"        => '20'
                );
                
                $sql = "INSERT INTO REGEVENREINF
                (NRCAMPO,NRREGISTRO, NREVENTO, NMCAMPO, NMCAMPOPAI,  DSELEMENTO, DSTIPO, DSOCORRENCIA, NRTAMANHO, NRDECIMAIS, DSREGISTRO, NRORG)
                        VALUES
                (:NRCAMPO, :NRREGISTRO,:NREVENTO,:NMCAMPO,:NMCAMPOPAI, :DSELEMENTO,:DSTIPO,:DSOCORRENCIA,:NRTAMANHO,:NRDECIMAIS, :DSREGISTRO, :NRORG)";
        
                $std = $connection->prepare($sql);
                $std->execute($params);
                \Setforms2\Shared\Registry::getEntityManager()->flush();

            }
        } catch (\Exception $e) {
            var_dump($e->getMessage());die;
            throw $e;
            
        }
    }

    //@todo Verificar possibilidade de organizar registros pais e filhos através da chave nrregistro criando um campo nrpai para gerar as tags do projects de forma correta
    //@todo pois quando há campos com o mesmo nome ocorrem erros que devem ser corrigidos manualmente.
    private function geraArvoreRegistro($versao, $connection)
    {

        // Busca o(s) evento(s) para fazer a geração
        
        $sql = "SELECT * FROM EVENTOREINF WHERE DSVERSAO = :DSVERSAO";
        $params = array(
            "DSVERSAO" => $versao
        );
        $std = $connection->prepare($sql);
        $std->execute($params);
        $eventos=  $std->fetchAll();

        foreach ($eventos as $evento) {
            $sql = " select * from REGEVENREINF where NREVENTO=:NREVENTO order BY NRCAMPO";
            $params = array(
                "NREVENTO" => $evento['NREVENTO']
            );
            $std = $connection->prepare($sql);
            $std->execute($params);
            $registrosOrdenados=  $std->fetchAll();
            foreach ($registrosOrdenados as $registro) {
                if ( $registro['DSELEMENTO'] ==='G'  ) {
                    $nrPai = $registro['NRCAMPO'];
                    
                    if (!is_null($registro['NMCAMPOPAI'])){
                        $sql = "UPDATE REGEVENREINF SET NRCAMPOPAI = 
                            ( SELECT MAX(NRCAMPO)
                                FROM REGEVENREINF A
                                WHERE dselemento = 'G'
                                AND NMCAMPO = :P_NMCAMPOPAI
                                AND NRCAMPO < :P_NRCAMPO
                            ) 
                        WHERE NRREGISTRO = :NRREGISTRO";
                        $params = array(
                        "NRREGISTRO"   => $registro['NRREGISTRO'],
                        "P_NMCAMPOPAI" => $registro['NMCAMPOPAI'],
                        "P_NRCAMPO"    => $nrPai
                        );
                        $std = $connection->prepare($sql);
                        $std->execute($params);
                        \Setforms2\Shared\Registry::getEntityManager()->flush();
                        
                    }
                   
                }
                else {
                    $sql = "UPDATE REGEVENREINF SET NRCAMPOPAI = :NRCAMPOPAI WHERE NRREGISTRO = :NRREGISTRO";
                    $params = array(
                        "NRREGISTRO" => $registro['NRREGISTRO'],
                        "NRCAMPOPAI" => $nrPai
                    );
                    $std = $connection->prepare($sql);
                    $std->execute($params);
                    \Setforms2\Shared\Registry::getEntityManager()->flush();
                }
            }
        }
    }
    
    
}

main();